// Listing 1 -- Sample.h : Definition of CSample

#include "resource.h"       // main symbols
#include <atlctl.h>

class ATL_NO_VTABLE CSample :
    public CComObjectRootEx<CComMultiThreadModel>,
    public IDispatchImpl<ISample, &IID_ISample, &LIBID_SAMPLEACTIVEXCONTROLLib>,
    public CComControl<CSample>,
    public IPersistStreamInitImpl<CSample>,
    public IOleControlImpl<CSample>,
    public IOleObjectImpl<CSample>,
    public IOleInPlaceActiveObjectImpl<CSample>,
    public IViewObjectExImpl<CSample>,
    public IOleInPlaceObjectWindowlessImpl<CSample>,
    public ISupportErrorInfo,
    public IConnectionPointContainerImpl<CSample>,
    public IPersistStorageImpl<CSample>,
    public ISpecifyPropertyPagesImpl<CSample>,
    public IQuickActivateImpl<CSample>,
    public IDataObjectImpl<CSample>,
    public IProvideClassInfo2Impl<&CLSID_Sample, &DIID__ISampleEvents,
                &LIBID_SAMPLEACTIVEXCONTROLLib>,
    public IPropertyNotifySinkCP<CSample>,
    public CComCoClass<CSample, &CLSID_Sample>,

    // Added to support property bag persistence:
    public IPersistPropertyBagImpl<CSample>
{
public:
    CSample()
    {
        m_bWindowOnly = TRUE;

        // Added to demonstrate using plain BSTR (as opposed to CComBSTR)
        // for string-valued property:
        #if USE_PLAIN_BSTR
            bstrSampleString = NULL;
        #endif

        // Added to demonstrate custom property persistence:
        plSampleArray = new long[1];
        plSampleArray[0] = 12345;
        countSampleArray = 1;

        // Added to demonstrate ambient properties:
        bAutomaticSIP = FALSE;
    }

    ~CSample()
    {
        // If we use CComBSTR, the destructor does this automatically...
        #if USE_PLAIN_BSTR
            if (bstrSampleString)
                SysFreeString(bstrSampleString);
        #endif

        // Free up our array-valued property.
        if (plSampleArray)
            delete [] plSampleArray;
    }

DECLARE_REGISTRY_RESOURCEID(IDR_SAMPLE)

DECLARE_PROTECT_FINAL_CONSTRUCT()

BEGIN_COM_MAP(CSample)
    COM_INTERFACE_ENTRY(ISample)
    COM_INTERFACE_ENTRY(IDispatch)
    COM_INTERFACE_ENTRY(IViewObjectEx)
    COM_INTERFACE_ENTRY(IViewObject2)
    COM_INTERFACE_ENTRY(IViewObject)
    COM_INTERFACE_ENTRY(IOleInPlaceObjectWindowless)
    COM_INTERFACE_ENTRY(IOleInPlaceObject)
    COM_INTERFACE_ENTRY2(IOleWindow, IOleInPlaceObjectWindowless)
    COM_INTERFACE_ENTRY(IOleInPlaceActiveObject)
    COM_INTERFACE_ENTRY(IOleControl)
    COM_INTERFACE_ENTRY(IOleObject)
    COM_INTERFACE_ENTRY(IPersistStreamInit)

    // added to support property bag persistence:
    COM_INTERFACE_ENTRY(IPersistPropertyBag)

    // changed to support property bag persistence:
    COM_INTERFACE_ENTRY2(IPersist, IPersistPropertyBag)

    COM_INTERFACE_ENTRY(ISupportErrorInfo)
    COM_INTERFACE_ENTRY(IConnectionPointContainer)
    COM_INTERFACE_ENTRY(ISpecifyPropertyPages)
    COM_INTERFACE_ENTRY(IQuickActivate)
    COM_INTERFACE_ENTRY(IPersistStorage)
    COM_INTERFACE_ENTRY(IDataObject)
    COM_INTERFACE_ENTRY(IProvideClassInfo)
    COM_INTERFACE_ENTRY(IProvideClassInfo2)
END_COM_MAP()

BEGIN_PROP_MAP(CSample)
    PROP_DATA_ENTRY("_cx", m_sizeExtent.cx, VT_UI4)
    PROP_DATA_ENTRY("_cy", m_sizeExtent.cy, VT_UI4)

    // added to make IPersistPropertyBagImpl<> persist this property:
    PROP_ENTRY("SampleString", // property name
                1, // dispid
                CLSID_NULL) // no custom property page
END_PROP_MAP()

BEGIN_CONNECTION_POINT_MAP(CSample)
    CONNECTION_POINT_ENTRY(IID_IPropertyNotifySink)
END_CONNECTION_POINT_MAP()

BEGIN_MSG_MAP(CSample)
    CHAIN_MSG_MAP(CComControl<CSample>)
    DEFAULT_REFLECTION_HANDLER()
END_MSG_MAP()

// ISupportsErrorInfo
    STDMETHOD(InterfaceSupportsErrorInfo)(REFIID riid)
    {
        if (InlineIsEqualGUID(IID_ISample, riid))
            return S_OK;
        else
            return S_FALSE;
    }

// IViewObjectEx
    DECLARE_VIEW_STATUS(VIEWSTATUS_SOLIDBKGND | VIEWSTATUS_OPAQUE)

// Drawing:
public:
    HRESULT OnDraw(ATL_DRAWINFO& di);

// Added to demonstrate string-valued properties:
public:
    STDMETHOD(get_SampleString)(BSTR *pVal);
    STDMETHOD(put_SampleString)(BSTR newVal);
private:
    #if USE_PLAIN_BSTR
        BSTR bstrSampleString;
    #else
        CComBSTR bstrSampleString;
    #endif

// Added to demonstrate array-valued properties:
public:
    STDMETHOD(get_SampleArray)(VARIANT *pVal);
    STDMETHOD(put_SampleArray)(VARIANT newVal);
private:
    long * plSampleArray;
    long countSampleArray;

// Added to demonstrate custom property bag persistence:
public:
    STDMETHOD(Save)(LPPROPERTYBAG pPropBag, BOOL fClearDirty,
                    BOOL fSaveAllProperties);
    STDMETHOD(Load)(LPPROPERTYBAG pPropBag, LPERRORLOG pErrorLog);

// Added to demonstrate raising custom errors:
public:
    STDMETHOD(RaiseError)(long number, BSTR message);

// Added to demonstrate using ambient properties:
public:
    STDMETHOD(OnAmbientPropertyChange)(DISPID dispid);
    STDMETHOD(SetClientSite)(IOleClientSite *pClientSite);
private:
    BOOL bAutomaticSIP;
    CComBSTR bstrDisplayName;
};
