/*****************************************************************
Title:  ImageWatch.java
Date:   12/5/00
Author: Michael Larson

Comments:
Streaming image implementation of the StreamWatch object.
*******************************************************************/

package streamhealth;

import java.io.*;
import java.net.*;
import java.util.*;
 
class ImageWatch extends StreamWatch
{

protected ImageWatch() {}
private boolean isGood(PrintStream ps, BufferedReader br, StringBuffer response) throws IOException {
	String s;
	boolean mFlg = false;
	
	//probe the server with DESCRIBE
	ps.println("HEAD " + mURL + " HTTP/1.1");
	ps.println("Accept: */*");
	ps.println("Accept-Language: en-us");
	ps.println("Accept-Encoding: gzip, deflate");
	ps.println("User-Agent: Mozilla/4.0 (compatible; MSIE 5.01; Windows NT 5.0; viper)");
	ps.println("Host: 127.0.0.1:80");
	ps.println(""); 

	//now let's the parse the output stream for the results of the signal
	if ((s = br.readLine()) != null) {
		response.append(s); //let's save this response for reporting
		if (s.equals("HTTP/1.1 100 CONTINUE") || 
		    s.equals("HTTP/1.1 200 OK") || 
		    s.equals("HTTP/1.1 201 Created")) {
		    mFlg = true; } }
	
	return mFlg;

}
public void run()
{
/*
	Streaming Image socket implementation, using HTTP protocol. This algorithm looks for time stamps
	associated with the image from the server, indicating it has been updated.
*/	
	Socket sock = null;
	BufferedReader inStream = null;
	PrintStream outStream = null;
	String s = null, sCode = null, tmpEq = null;
	StringBuffer response = null;
	int status = Konstants.kOPSTATUS_STREAM_BAD;
	ArrayList auxData = null;

	//no URL return as undefined and finish thread
	if (mURL == "") {
		status = Konstants.kOPSTATUS_STREAM_UNDEFINED;
		return; }

	while (true) {	//now we'll monitor this stream until doneWithStatus() returns false
		try {
			System.out.println(new Date().toString() + " starting to probe camera stream: " + mURL);
			response = new StringBuffer();
			sock = new Socket(mIP, mPort);
			System.out.println(new Date().toString() + " socket opened to server: " + mURL);
			inStream = new BufferedReader(new InputStreamReader(sock.getInputStream()));
			outStream = new PrintStream(sock.getOutputStream()); 		

			status = Konstants.kOPSTATUS_STREAM_BAD;
			if (isGood(outStream, inStream, response)) {

				while ((s = inStream.readLine()) != null) {
					if (!(s.equals(""))) {
						if (s.indexOf("Last-Modified:") >= 0) { //prefer "Last-Modified"
							sCode = s; 
							break; }
						if (s.indexOf("Content-Length:") >= 0) { //but we'll take "Content-Length"
							sCode = s; } } }

				tmpEq = new String(mPrevCode);
				if (!tmpEq.equals(sCode)) {
					mPrevCode = new StringBuffer(sCode);		//set this guy for next time through
					status = Konstants.kOPSTATUS_STREAM_OK; } } }	//code has changed since last check

		catch (IOException ioe) {
			System.out.println(new Date().toString() + " Exception thrown in ImageWatch.run() " + ioe);
			status = Konstants.kOPSTATUS_STREAM_BAD; } 
		try {
			if (sock != null) {
				sock.close(); }
			if (!mSiteState.doneWithStatus(mUniqueID, status, response.toString(), null) ) { 
				return; }	//I've been removed from monitoring
			sleep(Konstants.mDelayBetweenCycles.intValue()); }
		catch (InterruptedException ie) {
			System.out.println(new Date().toString() + " Exception thrown in ImageWatch.run() " + ie); } 
					catch (IOException ioe) {
			System.out.println(new Date().toString() + " Interrupted Exception thrown in ImageWatch.run() " + ioe); } } }
}
