/*****************************************************************
Title:  RealVideoWatch.java
Date:   12/5/00
Author: Michael Larson

Comments:
Stream RTSP video (only tested on RealServer) StreamWatch object.
Uses RTSP calls to verify stream supports describe and carries
stream on interest.
*******************************************************************/

package streamhealth;

import java.io.*;
import java.net.*;
import java.util.*;

class RealVideoWatch extends StreamWatch
{
protected RealVideoWatch() {}
public void run()
{
/*
	Checks on an RTSP streaming server for the existance of the DESCRIBE method, 
	then probes for the existance of the live stream.
*/	
	Socket sock = null;
	BufferedReader inStream = null;
	PrintStream outStream = null;
	String s;
	StringBuffer response = null;
	int status = Konstants.kOPSTATUS_STREAM_UNDEFINED;
	ArrayList addStatus = null;
	
	//this indicates a faulty streamlist.xml entry		
	if (mURL == "") {
		return; }
		
	while (true) { //now we'll monitor this stream until doneWithStatus() returns false
		System.out.println(new Date().toString() + " starting to probe camera stream: " + mURL);
		response = new StringBuffer();
		
		try { 
			sock = new Socket(mIP, mPort);
			System.out.println(new Date().toString() + " socket opened to server: " + mURL);
			inStream = new BufferedReader(new InputStreamReader(sock.getInputStream()));
			outStream = new PrintStream(sock.getOutputStream()); 

			//now let's start communicating with the RTSP server		
			if (supportsDescribe(outStream, inStream)) {	
				if (isGood(outStream, inStream, response)) {						
					status = Konstants.kOPSTATUS_STREAM_OK;
					System.out.println(new Date().toString() + " Stream is good: " + mURL);
					//let's see if we can capture some additional details
					addStatus = new ArrayList();		//initialize for loop						
					getAuxData(outStream, inStream, addStatus); }			
				else {	//stream not available from RTSP server
					status = Konstants.kOPSTATUS_STREAM_BAD;
					addStatus = null;
					System.out.println(new Date().toString() + " Stream is BAD: " + mURL); } } }
		catch (IOException ioe) {
			status = Konstants.kOPSTATUS_STREAM_BAD;
			System.out.println(new Date().toString() + " IO Exception thrown in RealVideoWatch.run() " + ioe); }

		try {
			if (sock != null) {
				sock.close(); }
			if (!mSiteState.doneWithStatus(mUniqueID, status, response.toString(), addStatus) ) { 
				return; } //I've been removed from monitoring

			sleep(Konstants.mDelayBetweenCycles.intValue()); }
		catch (InterruptedException ie) {
			System.out.println(new Date().toString() + " Interrupted Exception thrown in RealVideoWatch.run() " + ie); }
		catch (IOException ioe) {
			System.out.println(new Date().toString() + " Interrupted Exception thrown in RealVideoWatch.run() " + ioe); }
	}
}

private void getAuxData(PrintStream ps, BufferedReader br, ArrayList al) throws IOException {
	String s;
	int len;
	
	//probe the server with GET_PARAMETER
	ps.println("GET_PARAMETER " + mURL + " RTSP/1.0");
	ps.println(""); 

	//now let's capture some aux data that might be interesting
	while (!((s = br.readLine()).equals(""))) {
	//	System.out.println(s);
		if ((len = s.indexOf("a=AvgBitRate:")) >= 0) {
			al.add(new String(s));}
		else if ((len = s.indexOf("a=LiveStream:")) >= 0) {
			al.add((new String(s)).substring(len,len+20));}
		else if ((len = s.indexOf("a=MaxBitRate:")) >= 0) {
			al.add(new String(s));}
		else if ((len = s.indexOf("a=Target Audiences:")) >= 0) {
			al.add((new String(s)).substring(len,len+20));}
		else if ((len = s.indexOf("a=ASMRuleBook:")) >= 0) {
			al.add(new String(s));} }
	
}

private boolean isGood(PrintStream ps, BufferedReader br, StringBuffer response) throws IOException {
	String s;
	boolean mFlg = false;
	
	//probe the server with DESCRIBE
	ps.println("DESCRIBE " + mURL + " RTSP/1.0");
	ps.println(""); 

	//now let's the parse the output stream for the results of the signal
	if ((s = br.readLine()) != null) {
		response.append(s); //let's save this response for reporting
		if (s.equals("RTSP/1.0 100 CONTINUE") || 
		    s.equals("RTSP/1.0 200 OK") || 
		    s.equals("RTSP/1.0 201 Created") || 
		    s.equals("RTSP/1.0 250 Low on Storage Space")) {
		    mFlg = true; } }
	
	while (!(br.readLine()).equals("")) { }

	return mFlg;
}

private boolean supportsDescribe(PrintStream ps, BufferedReader br) throws IOException {
	String s;
	boolean mFlg = false;

	//let's see what the RTSP server supports
	ps.println("OPTIONS * RTSP/1.0");
	ps.println("");

	while (!(s = br.readLine()).equals("")) {
		if (s.indexOf("DESCRIBE") >= 0) {
			//we do support DESCRIBE
			mFlg = true; 
			break; } }

	//flush out streawm
	while (!(br.readLine()).equals("")) { } 
			
	return mFlg;
}}
