/*****************************************************************
Title:  ReportMgr.java
Date:   12/5/00
Author: Michael Larson

Comments:
Manages the reporting objects. Expects to be woken up by the 
SiteState object to consume events.
*******************************************************************/

package streamhealth;

import java.util.*;
 
public class ReportMgr extends Thread {

private ReportDIR mReportDIR;
private ReportHTML mReportHTML;
private SiteState mSiteState;


public ReportMgr(SiteState sState) {
	mSiteState = sState;

	mReportDIR = new ReportDIR();
	mReportHTML = new ReportHTML(); }

private void finishUpdate(ArrayList aList) {
	mReportDIR.finishUpdate(aList);
	mReportHTML.finishUpdate(aList); }

private void reportGeneralError() {
	mReportDIR.reportGeneralError();
	mReportHTML.reportGeneralError();	}

public void run() {
	StatusAList aList;

	System.out.println(new Date().toString() + " ReportMgr::run() running");
	
	try {
		while(true) {
			while ((aList = (StatusAList)mSiteState.getUpdateBuf()) != null) {
				switch (aList.mStatus) {
					case Konstants.kSTATUS_HAS_STARTED:
						startUpdate((ArrayList)aList);
						break;
					case Konstants.kSTATUS_IS_UPDATED: 
						update((ArrayList)aList);
						break;
					case Konstants.kSTATUS_IS_COMPLETE:
						finishUpdate((ArrayList)aList);
						break;
					case Konstants.kSTATUS_NO_STREAMS:
						reportGeneralError(); 
						break;
					default:
						System.out.println(new Date().toString() + " Bad Status flag in  ReportMgr:run() "); } }
			synchronized(mSiteState) {
				mSiteState.wait(); } } }  //now let's wait for an event to be posted.
			catch (InterruptedException ie) {
				System.out.println(new Date().toString() + " InterruptedException thrown in  ReportMgr:run() " + ie);  } 
			catch (ClassCastException cce) { 
				System.out.println(new Date().toString() + " ClassCastException thrown in  ReportMgr:run() " + cce);  } } 

private void startUpdate(ArrayList aList) {
	mReportDIR.startUpdate(aList);
	mReportHTML.startUpdate(aList); }

private void update(ArrayList aList) {
	mReportDIR.update(aList);
	mReportHTML.update(aList); }
}
