/*****************************************************************
Title:  SiteState.java
Date:   12/5/00
Author: Michael Larson

Comments:
SiteState object that coordinates the checking of the video streams.
This also keeps the internal state of the streams being actively
monitored
*******************************************************************/

package streamhealth;

import java.io.*;
import java.net.*;
import java.util.*;
import org.xml.sax.Parser;
import org.xml.sax.helpers.ParserFactory; 
import org.w3c.dom.Node;
import com.ibm.xml.parsers.DOMParser; 
import org.w3c.dom.Document;
import org.xml.sax.SAXException;

import java.io.IOException; 
import java.io.UnsupportedEncodingException;
import org.w3c.dom.*;

public class SiteState extends Thread {

private Vector mStreamVector;		//contains a vector of the streams that need to be tested
private StreamData mCurStreamData;	//current streamdata object that is being worked on
private StreamWatchFactory mStreamWatchFactory; //factory to create streamwatch objects
private StreamFile mStreamFile;    //internal stream file
private Vector mSDBufferVect;		//report buffer


private boolean reconcileNewStreamConfigData(Vector newStreamVect) {
	Iterator newIter, oldIter;
	boolean bMatch;
	StreamData oldSD = null, newSD = null;
	
	try {
		//1) compare new against old and look for new vectors
		oldIter = mStreamVector.iterator();
		while (oldIter.hasNext()) {
			bMatch = false;
			oldSD = (StreamData)oldIter.next();
			newIter = newStreamVect.iterator();
			while (newIter.hasNext()) {
				newSD = (StreamData)newIter.next();
				if (newSD.equals(oldSD)) {
					bMatch = true; 
					break; } }
			if (bMatch == false) {		//didn't find a match in the old StreamVector, let's add it then and mark it
				oldSD.mState = Konstants.kSTATE_DELETED;  } }

		//2) compare old against new and look for old vectors
		newIter = newStreamVect.iterator();
		while (newIter.hasNext()) {
			bMatch = false;
			newSD = (StreamData)newIter.next();
			oldIter = mStreamVector.iterator();
			while (oldIter.hasNext()) {
				oldSD = (StreamData)oldIter.next();
				if (oldSD.equals(newSD)) {
					bMatch = true;
					break; } }
			if (bMatch == false) {		
				newSD.mState = Konstants.kSTATE_ADDED; 
				newSD.mSW = (StreamWatch)mStreamWatchFactory.getStreamWatch(newSD.mType.toString());
				mStreamVector.add(newSD); } } } //we have an outdated stream, let's kick it on out
	catch (ConcurrentModificationException cme) {
		System.out.println(new Date().toString() + " Exception thrown in SiteState.reconcileNewStreamConfigData() " + cme); }			
	return true;
}

    
//internal file parsing class
public class StreamFile {

private String mStreamDataFile;


public StreamFile(String dataFile) throws FileNotFoundException {
	if (dataFile.equals(null)) {
		System.out.println(new Date().toString() + "StreamData::constructor null datafile passed");
		throw new FileNotFoundException(); }
	mStreamDataFile = dataFile; }


public Vector readConfigFile(Vector streamVect) throws RunTimeException
{
	StreamData sd;
	Iterator iter;
	String parserClass = "com.ibm.xml.parsers.DOMParser";
	DOMParser parser = null;
	try { 
		System.out.println(new Date().toString() + " StreamFile.readConfigFile(): Parsing StreamMon Configuration file");
		parser = (DOMParser)ParserFactory.makeParser( parserClass );
		parser.parse(Konstants.mStreamDataFile); } 
   
  	catch (SAXException se) {
		System.out.println(new Date().toString() + " StreamFile.readConfigFile(): SAXException thrown " + se);
	  	throw new RunTimeException(); }
  	catch (IOException ioe) {
		System.out.println(new Date().toString() + " StreamFile.readConfigFile(): IOException thrown " + ioe); 
	  	throw new RunTimeException(); }
  	catch (ClassNotFoundException cnfe ) {
		System.out.println(new Date().toString() + " StreamFile.readConfigFile(): ClassNotFoundException thrown " + cnfe);  
	  	throw new RunTimeException(); }
  	catch (IllegalAccessException iae ) {
		System.out.println(new Date().toString() + " StreamFile.readConfigFile(): IllegalAccessException thrown " + iae);  
	  	throw new RunTimeException(); }
  	catch (InstantiationException ie ) {
		System.out.println(new Date().toString() + " StreamFile.readConfigFile(): InstantiationException thrown " + ie);  
	  	throw new RunTimeException(); }
  	catch (ClassCastException cce ) {
		System.out.println(new Date().toString() + " StreamFile.readConfigFile(): ClassCastException thrown " + cce);  
	  	throw new RunTimeException(); }  	
	// The next lines are only for DOM Parsers 
	Document doc = ((DOMParser) parser).getDocument();

	if ( doc != null ) {
	traverse((Node)doc, streamVect); } //here is where we traverse the nodes

	//need to perform some validation of the object created.
	iter = streamVect.iterator();
	while (iter.hasNext()) {
		sd = (StreamData)iter.next();
		sd.validate(); }
	
return (Vector) streamVect; }
private boolean traverse(Node node, Vector streamVect) 
{
	String nName;
	NodeList children = null;
	Node nodeAttr = null, cNode = null;
	NamedNodeMap attrs = null;
	int i, len;
	StreamData streamData = null;
	
	// is there anything to do?
	if (node == null) {
		return false; }

	int type = node.getNodeType();
	switch (type) {
		// print document
		case Node.DOCUMENT_NODE:
			traverse(((Document)node).getDocumentElement(), streamVect);
			break;
		case Node.TEXT_NODE:
			break;
		case Node.ENTITY_REFERENCE_NODE:
		case Node.ELEMENT_NODE:
			nName = node.getNodeName();
			if (nName.equals("Stream")) {
				attrs = node.getAttributes();
				if (attrs != null) {
					len = attrs.getLength();
					for (i = 0; i < len; i++) {
						//compare here and assign here
						nodeAttr = attrs.item(i);
						if (nodeAttr.getNodeName().equals("Type")) {
							//create a new StreamData object right here then
							streamData = new StreamData();
							streamVect.add(streamData);
							if (nodeAttr.getNodeValue().equals(Konstants.kTYPE_STREAM_IMAGE)) {
								streamData.mType = new StringBuffer(Konstants.kTYPE_STREAM_IMAGE); }
							else if (nodeAttr.getNodeValue().equals(Konstants.kTYPE_STREAM_VIDEO)) {
								streamData.mType = new StringBuffer(Konstants.kTYPE_STREAM_VIDEO); }
							else {
								streamData.mType = new StringBuffer(Konstants.kTYPE_STREAM_UNDEFINED);
								System.out.println(new Date().toString() + " StreamFile::traverse() bad stream type found in xml file: " + nodeAttr.getNodeValue()); } } } } }
			else if (nName.equals("Location")) {
				attrs = node.getAttributes();
				if (attrs != null) {
					len = attrs.getLength();
					for (i = 0; i < len; i++) {
						//compare here and assign here
						nodeAttr = attrs.item(i);
						if (nodeAttr.getNodeName().equals("Path")) {
							streamData = (StreamData)streamVect.lastElement();
							if (streamData != null) {
								streamData.mURL = nodeAttr.getNodeValue(); } } } } }
			else if (nName.equals("Server")) {
				attrs = node.getAttributes();
				if (attrs != null) {
					len = attrs.getLength();
					for (i = 0; i < len; i++) {
						nodeAttr = attrs.item(i);
						if (nodeAttr.getNodeName().equals("IP")) {
							streamData = (StreamData)streamVect.lastElement();
							if (streamData != null) {
								streamData.mServer = nodeAttr.getNodeValue(); } }
						else if (nodeAttr.getNodeName().equals("Port")) {
							streamData = (StreamData)streamVect.lastElement();
							if (streamData != null) {
								streamData.mPort = new Integer(nodeAttr.getNodeValue()).intValue(); } } } } }
			else {
					/*don't do anything here*/ }
			
			children = node.getChildNodes();
			if ( children != null ) {
				len = children.getLength();
				for (i = 0; i < len; i++) {
					traverse(children.item(i), streamVect); } }
			break;
		case Node.CDATA_SECTION_NODE: {
			break; } }			

	return true;
}
}

  
public SiteState(StreamWatchFactory swFactory) {

	mStreamWatchFactory = swFactory;
	
	//init data file parser object
	try {
		mStreamFile = new StreamFile(Konstants.mStreamDataFile); }
	catch (FileNotFoundException fnfe) {
		System.out.println(new Date().toString() + "Exception thrown in SiteState.SiteState() " + fnfe); }

	//init the internal vector object
	mStreamVector = new Vector();

	mSDBufferVect = new Vector();
}  

private void finishUpdate() {
	Iterator iter;
	StreamData sd;
	StatusAList endAList;
	
	endAList = new StatusAList();
	//now run through and cull the list of streamdata entries that are now found in the current configuration file
	iter = mStreamVector.iterator();
	while (iter.hasNext()) {
		sd = (StreamData)iter.next();
		if ((sd.mState == Konstants.kSTATE_DELETED)) {
			iter.remove(); } 
		else {
			endAList.add(new StreamData(sd)); } }	//copy the data to the vector to be handed off and notify

	endAList.mStatus = Konstants.kSTATUS_IS_COMPLETE;
	postUpdate(endAList);
}

private void checkStreams() {
	StreamData sd = null;
	StatusAList aList;
	Iterator iter;
	
	iter = mStreamVector.iterator();	
	//move to the next stream in sequence
	try {
		while (iter.hasNext()) {
			sd = (StreamData)iter.next();
			if (sd.mState == Konstants.kSTATE_ADDED) {
				sd.mState = Konstants.kSTATE_UPDATED;
				sd.mSW.init(sd, sd.mURL, sd.mServer, sd.mPort, sd.mCode, this); //pass in the url, but use the sd as a reference to the original object (i.e. no locking)
				sd.mSW.start(); }
			
			sleep(Konstants.mDelayBetweenStreams.intValue()); } }  //wait an increment in time before launching the next sock thread
	catch (InterruptedException e) {
				System.out.println(new Date().toString() + "Exception thrown in SiteState.checkNextStream() " + e); }
	catch (NullPointerException e) {
				System.out.println(new Date().toString() + "Exception thrown in SiteState.checkNextStream() " + e); }
	return;
}

public boolean doneWithStatus(Object sdobj, int status, String resp, ArrayList auxData) {
	boolean bChanged = false;
	StatusAList updateAList;
	StreamData sd;
	
	sd = (StreamData)sdobj;
	sd.mTimeOfLastTest = new Date().toString();		
	//find the stream then update and notify if status or server response has changed
	if (mStreamVector.contains(sd)) {
		if (sd.mOpStatus != status || sd.mServerResponse != resp) {
			sd.mState = Konstants.kSTATE_UPDATED; 
			bChanged = true; }
		else {
			sd.mState = Konstants.kSTATE_UNCHANGED; }
		
		sd.mOpStatus = status;
		sd.mServerResponse = resp;
		sd.mAddConnInfo = auxData;

		if (bChanged == true) {		//update if status has changed
			updateAList = new StatusAList();
			updateAList.add(new StreamData(sd));
			updateAList.mStatus = Konstants.kSTATUS_IS_UPDATED;
			postUpdate(updateAList); }
		
		return true; }		//this stream is still actively monitored
	else {
		return false;}		//didn't find match is deleted
}

public StatusAList getUpdateBuf() {
	StatusAList statAList = null;

	try {	//only called by ReportMgr.run() method
		if (!mSDBufferVect.isEmpty()) {
			statAList = (StatusAList) mSDBufferVect.get(0); //get head element
			mSDBufferVect.remove(statAList); } }					
	catch (NoSuchElementException e) {	
		statAList = null; }

	return statAList;
}

private void postUpdate(StatusAList aList) {

	mSDBufferVect.add(aList); 
	synchronized (this) {
		notifyAll(); } //wake up at the beginning of a cycle
}

public void run() {
	StatusAList aList;
	
	System.out.println(new Date().toString() + " SiteState::run() running");
	
	try {
		while (true) {
			try {
				startUpdate();	//init update cycle

				checkStreams();	//start checking individual streams and reporting

				finishUpdate(); }	//wrapup update cycle
			catch (RunTimeException rte) {
				aList = new StatusAList();	//report that we had a runtime exception thrown
				aList.mStatus = Konstants.kSTATUS_NO_STREAMS;
				postUpdate(aList);
				System.out.println(new Date().toString() + " RunTimeException thrown in SiteState:run() " + rte);}	
			//now sleep until the next check		
			sleep(Konstants.mDelayBetweenCycles.intValue()); } }
	catch (InterruptedException ie) {
		System.out.println(new Date().toString() + " Exception thrown in SiteState:run() " + ie); } }private void startUpdate() throws RunTimeException {
	Vector streamReadVect = null;
	StatusAList startAList = null;
	StreamData sd;
	Iterator iter;
		
	streamReadVect = new Vector();

	//parse and populate streamRecVect with current streamlist
	mStreamFile.readConfigFile(streamReadVect);
	if (Konstants.mNotifyOfNoStreams && streamReadVect.isEmpty()) {
		throw new RunTimeException(); }

	//reconcile against internal vector
	reconcileNewStreamConfigData(streamReadVect);

	iter = mStreamVector.iterator();

	//copy the data to the vector to be handed off and notify
	startAList = new StatusAList();
	while (iter.hasNext()) {
		sd = (StreamData)iter.next();
		startAList.add(new StreamData(sd)); }

	startAList.mStatus = Konstants.kSTATUS_HAS_STARTED;
	postUpdate(startAList);	
	}}
