/*****************************************************************
Title:  StreamHealth.java
Date:   12/5/00
Author: Michael Larson

Comments:
StreamHealth is the main entry point for this service. Launches the
two main threads: ReportMgr, and SiteState.
*******************************************************************/

package streamhealth;

import java.io.*;
import java.net.*;
import java.util.*;

import org.xml.sax.Parser;
import org.xml.sax.helpers.ParserFactory; 
import org.w3c.dom.Node;
import com.ibm.xml.parsers.DOMParser; 
import org.w3c.dom.Document;
import org.xml.sax.SAXException;

import java.io.IOException; 
import java.io.UnsupportedEncodingException;
import org.w3c.dom.*;

public class StreamHealth {

public StreamHealth() {super();}

public static void main(String[] args) {
	SiteState siteState = null;
	ReportMgr reportMgr = null;
	
	if (args.length != 1) {
		System.out.println("Usage: StreamHealth configFile");}
	
	System.out.println(new Date().toString() + " StreamHealth::main(): started");
	//the main entry point for the monitor stream
 
	Konstants.mStreamHealthConfigFile = args[0];

	//initialize and read configuration file for StreamMon constants to be read in via the file
	(new StreamHealthConfig()).readConfigFile();
	
	//initialize the configuration object
	siteState = new SiteState(new StreamWatchFactory());

	//initialize the report object
	reportMgr = new ReportMgr(siteState);

	System.out.println(new Date().toString() + " StreamMonitor::run() running");

	siteState.start();		//spin off the stream checking thread

	reportMgr.start();		//spin off reporting thread
}

    //internal class to parse config file
static public class StreamHealthConfig {
private StreamHealth mStreamHealth;
public StreamHealthConfig() {super();}

public boolean readConfigFile() 
	{
	   String parserClass = "com.ibm.xml.parsers.DOMParser";
	   DOMParser parser = null;
	   try { 
			System.out.println(new Date().toString() + " Parsing StreamMon Configuration file");


			parser = (DOMParser)ParserFactory.makeParser( parserClass );
			parser.parse(Konstants.mStreamHealthConfigFile);
 
	   } catch (SAXException se) { 
			 se.printStackTrace(); 
	   } catch (IOException ioe) { 
			 ioe.printStackTrace(); 
	   } catch (ClassNotFoundException ex ){
			 ex.printStackTrace();
	   } catch (IllegalAccessException ex ) {
			 ex.printStackTrace();
	   } catch (InstantiationException ex ) {
			 ex.printStackTrace();
	   } catch (ClassCastException ex ) {
			 ex.printStackTrace();
	   } 

// The next lines are only for DOM Parsers 
	Document doc = ((DOMParser) parser).getDocument();
	if ( doc != null ) {
			traverse((Node)doc); }
	return true;
	}

private boolean traverse(Node node) 
{
	String nName;
	NodeList children = null;
	Node nodeAttr = null, cNode = null;
	NamedNodeMap attrs = null;
	int i, len;

	// is there anything to do?
	if (node == null) {
		return false; }

	int type = node.getNodeType();
	switch (type) {
		// print document
		case Node.DOCUMENT_NODE: 
			traverse(((Document)node).getDocumentElement());
			break; 
		case Node.TEXT_NODE:
			break;
		case Node.ENTITY_REFERENCE_NODE:
		case Node.ELEMENT_NODE:
			nName = node.getNodeName();
			if (nName.equals("StreamData")) {
				attrs = node.getAttributes();
				if (attrs != null) {
					len = attrs.getLength();
					for (i = 0; i < len; i++) {
						//compare here and assign here
						nodeAttr = attrs.item(i);
						if (nodeAttr.getNodeName().equals("File")) {
							Konstants.mStreamDataFile = new String(nodeAttr.getNodeValue()); } } } }
			else if (nName.equals("StreamReport")) {
				attrs = node.getAttributes();
				if (attrs != null) {
					len = attrs.getLength();
					for (i = 0; i < len; i++) {
						//compare here and assign here
						nodeAttr = attrs.item(i);
						if (nodeAttr.getNodeName().equals("Path")) {
							Konstants.mReportDir = new String(nodeAttr.getNodeValue()); } } } }
			else if (nName.equals("ConnCycle")) {
				attrs = node.getAttributes();
				if (attrs != null) {
					len = attrs.getLength();
					for (i = 0; i < len; i++) {
						//compare here and assign here
						nodeAttr = attrs.item(i);
						if (nodeAttr.getNodeName().equals("StreamDelay")) {
							Konstants.mDelayBetweenStreams = new Integer(nodeAttr.getNodeValue()); }
						else if (nodeAttr.getNodeName().equals("StatusDelay")) {
							Konstants.mDelayBetweenCycles = new Integer(nodeAttr.getNodeValue()); } } } }			
			else if (nName.equals("StreamHTMLReport")) {
				attrs = node.getAttributes();
				if (attrs != null) {
					len = attrs.getLength();
					for (i = 0; i < len; i++) {
						//compare here and assign here
						nodeAttr = attrs.item(i);
						if (nodeAttr.getNodeName().equals("File")) {
							Konstants.mHTMLReportStatusDoc = new String(nodeAttr.getNodeValue()); } } } }
			else if (nName.equals("Pulse")) {
				attrs = node.getAttributes();
				if (attrs != null) {
					len = attrs.getLength();
					for (i = 0; i < len; i++) {
						//compare here and assign here
						nodeAttr = attrs.item(i);
						if (nodeAttr.getNodeName().equals("Frequency")) {
							Konstants.mDelayBetweenCycles = new Integer(nodeAttr.getFirstChild().getNodeValue()); } } } }
			else { }
			children = node.getChildNodes();
			if ( children != null ) {
				len = children.getLength();
				for (i = 0; i < len; i++) {
					traverse(children.item(i)); } }
			break;
		case Node.CDATA_SECTION_NODE: {
			break; } }

	return true;
}
}


}
