// *****
// * PROJECT:		Memory utilities
// * FILENAME: 		MemDbg.h
// * AUTHOR:		Jeff Ishaq 05/12/00 jishaq@earthlink.net
// * 
// * DESCRIPTION:	Interface for some memory testing tools.
// *
// * COPYRIGHT:  	With the exception of this comment block, you are free to modify
// *				and/or to reuse this source code as you see fit.  For the sake
// *				of attribution and documentation, I ask that you leave this base
// *				comment block intact, though freely extend it.
// *****

#pragma once
#include "MyPilot.h"

// This defines the database we use to store the Nodes.  Each Node contains information
// about each allocation.  There is one record per node.
#define MemDbgDBCardNum		0
#define MemDbgDBName		"MemDbgDB"
#define MemDbgCreatorID		'jeff'
#define MemDbgType			'data'
#define MemDbgFilenameSize	19				// Size of the filename string in a node

// This defines the padding that goes at the end of allocations, used for testing
// boundary overruns.  It's very unlikely that a chunk will just happen to be
// overwritten with four ~'s, so this mechanism has a very high chance always to work.
#define MemDbgPaddingLen	4
#define MemDbgPaddingChar	'~'

// Each node knows if it came fro MemHandleNew, or MemPtrNew.  
typedef enum tageAllocationType { PointerType, HandleType } eAllocationType;

// Each node contains information about the allocation:
typedef struct tagNode
{
	// Constructor does cleanup:
	tagNode()		{ 
						MemSet( fileName, MemDbgFilenameSize, NULL);
						size 	= 0; 
						lineNum = 0; 
						pAddr 	= NULL; 
					}
					
	// Store the name and line number where the allocation happened, and also
	// the size and type of allocation.  Finally, store the actual memory address.
	Char 			fileName[MemDbgFilenameSize+1];		// Name of the file of allocation
	ULong			size;				
	ULong 			lineNum;
	eAllocationType type;
	union 
	{
		VoidPtr		pAddr;
		VoidHand	hAddr;
	};
} Node;
typedef Node* NodePtr;

// These are file-global vars used for database stuff:
static 	LocalID				MemDbgDBID 	= NULL;
static 	DmOpenRef			MemDbgRef	= NULL;

// These are also file-global vars, for system trap stuff:
typedef Err (*myMemHandleFreePtr)(VoidHand h );
static myMemHandleFreePtr			gpOldMemHandleFree = NULL;

typedef Err (*myMemChunkFreePtr)(VoidPtr p );
static myMemChunkFreePtr			gpOldMemChunkFree = NULL;


// Function Prototypes:
VoidPtr		operator new( UInt32 size, CharPtr pFileName, ULong lineNum );
VoidPtr		operator new[] ( UInt32 size, CharPtr pFileName, ULong lineNum );
void 		_AddDbgEntry( NodePtr pNode );
void 		AddDbgEntry( CharPtr pFileName, ULong size, ULong lineNum, VoidPtr pAddr );
void 		AddDbgEntry( CharPtr pFileName, ULong size, ULong lineNum, VoidHand hAddr );
void 		CheckForLeaks();
VoidHand	_DbgMemHandleNew( UInt size, CharPtr pFileName, ULong lineNum );
Err		 	_DbgMemHandleFree( VoidHand h );
Err			_DbgMemChunkFree( VoidPtr p );
VoidPtr		_DbgMemPtrNew( UInt size, CharPtr pFileName, ULong lineNum );
UInt 		_FindDbgEntry( VoidPtr addr, eAllocationType type );
UInt 		FindDbgEntry( VoidPtr	pAddr );
UInt 		FindDbgEntry( VoidHand hAddr );
void		HookMemMgr();
void 		KillRecord( UInt i );
void 		MakeMemDbgDB();
void 		_RemoveDbgEntry( VoidPtr addr, eAllocationType type );
void 		RemoveDbgEntry( VoidPtr pAddr );
void 		RemoveDbgEntry( VoidHand hAddr );
void 		UnhookMemMgr();