// *****
// * PROJECT:		Memory utilities
// * FILENAME: 		build.h
// * AUTHOR:		Jeff Ishaq 04/27/01 jishaq@earthlink.net
// * 
// * DESCRIPTION:	This is the 'parsed before any other .h files are parsed' .h file.
// *                In Codewarrior, each target can point to a different prefix file 
// *                (this file).  See Project Settings : Language Settings : C/C++  
// *                Language : Prefix File
// * 
// * NOTES:         I generally find that precompiled headers are more trouble than
// *                they are worth, so as a rule of thumb, my prefix file always turns
// *                them off first thing.  
// *
// * COPYRIGHT:  	With the exception of this comment block, you are free to modify
// *				and/or to reuse this source code as you see fit.  For the sake
// *				of documentation, I ask that you leave this base comment block
// *                intact, and I encourage you to extend it as necesary.
// *
// * REVISIONS:
// *    JeffI       04/27/01    Tidied this up in preparation for publication
// *****

// Shut off precompiled headers if someone else didn't already do this:
#ifndef PILOT_PRECOMPILED_HEADERS_OFF
	#define PILOT_PRECOMPILED_HEADERS_OFF
#endif

// This forces ErrDisplay and ErrFatalDisplayIf calls to become compiled into the build.
// This is important because it's the mechanism by which we display memory leak msgboxes!
#define ERROR_CHECK_LEVEL ERROR_CHECK_FULL

// This #define enable memory leak detection; comment it out  to 'turn it off' across
// the entire source code project over which this prefix file resides.  If you are having
// problems with this 'universal' definition, the unpleasant alternative is to add the two
// #include's at the top of EVERY source code file in your project in which you'd like to
// check for memory leaks.  Sometimes, this is actually desired -- say, if you suspect a leak
// in a particular set of files or something.

// NOTE:  This should only be on for in-house debug builds!  Never leave this on for production
// builds!  This causes traps to be patched, which can make the OS unstable.

#define CHECK_FOR_MEMORY_LEAKS

#ifdef CHECK_FOR_MEMORY_LEAKS
   #include "MemDbg.h"
   #include "MemDbgMacros.h"       // Turns on macro remapping of MemHandleNew() and similar  
                                   // functions at the very highest level.  This is the same
                                   // as #including this file in every single source code
                                   // file in your project.
#endif // CHECK_FOR_MEMORY_LEAKS
